/*
 * $Id: radacct.c,v 1.11 1996/09/15 11:36:36 lf Exp $
 *
 * Copyright (C) 1995,1996 Lars Fenneberg
 *
 * See the file COPYRIGHT for the respective terms and conditions. 
 * If the file is missing contact me at lf@elemental.net 
 * and I'll send you a copy.
 *
 */

static char	rcsid[] =
		"$Id: radacct.c,v 1.11 1996/09/15 11:36:36 lf Exp $";

#include	<sys/types.h>
#include	<sys/socket.h>
#include	<sys/param.h>
#include	<netinet/in.h>
#include	<sys/time.h>
#include	<sys/termios.h>
#include	<sys/signal.h>

#include	<stdio.h>
#include	<netdb.h>
#include	<unistd.h>
#include	<pwd.h>
#include	<string.h>
#include	<stdlib.h>
#include	<ctype.h>
#include	<syslog.h>

#include	"config.h"
#include	"radius.h"
#include	"messages.h"
#include	"pathnames.h"

static char *pname;

void usage(void)
{
	fprintf(stderr,"Usage: %s [-Vh] [-i <client_port>]\n\n", pname);
	fprintf(stderr,"  -i            ttyname to send to the server\n");
	fprintf(stderr,"  -V            output version information\n");
	fprintf(stderr,"  -h            output this text\n");
	exit(ERROR_RC);
}

void version(void)
{
	fprintf(stderr,"%s: %s\n", pname ,rcsid);
	exit(ERROR_RC);
}

int
main (int argc, char **argv)
{
	int			result = ERROR_RC;
	VALUE_PAIR	*send = NULL;
   	UINT4		client_port = 0;
    int			c;
    VALUE_PAIR	*vp;
    DICT_VALUE  *dval;
	char *username, *service, *fproto, *type;

    extern char *optarg;
    extern int optind;

	pname = (pname = strrchr(argv[0],'/'))?pname+1:argv[0];

	rc_openlog(pname);

	if (rc_read_config(_PATH_RADIUSCLIENT_CONF) != 0)
		exit(ERROR_RC);
	
	if (rc_read_dictionary(rc_conf_str("dictionary")) != 0)
		exit (ERROR_RC);

    if (rc_read_mapfile(rc_conf_str("mapfile")) != 0)
    	exit (ERROR_RC);


	while ((c = getopt(argc,argv,"i:hV")) > 0)
	{
		switch(c) {
			case 'i':
				client_port = rc_map2id(optarg);
				break;
			case 'V':
				version();
				break;
			case 'h':
				usage();
				break;
			default:
				exit(ERROR_RC);
				break;
		}
	}
	
	if (client_port == 0) {
		/* we take stdout here, because stdin is usually connected
	 	* to our input file
	 	*/
		client_port = rc_map2id(ttyname(1));
	}

	if ((send = rc_avpair_readin(stdin))) {

		username = service = type = "(unknown)";
		fproto = NULL;
	
		if ((vp = rc_avpair_get(send, PW_ACCT_STATUS_TYPE)) != NULL)
				if ((dval = rc_dict_getval(vp->lvalue, vp->name)) != NULL) {
					type = dval->name;
				}

		if ((vp = rc_avpair_get(send, PW_USER_NAME)) != NULL)
				username = vp->strvalue;

		if ((vp = rc_avpair_get(send, PW_SERVICE_TYPE)) != NULL)
				if ((dval = rc_dict_getval(vp->lvalue, vp->name)) != NULL) {
					service = dval->name;
				}

		if (vp && (vp->lvalue == PW_FRAMED) &&
			((vp = rc_avpair_get(send, PW_FRAMED_PROTOCOL)) != NULL))
				if ((dval = rc_dict_getval(vp->lvalue, vp->name)) != NULL) {
					fproto = dval->name;
				}

		result = rc_acct(client_port, send);
		if (result == OK_RC)
		{
			fprintf(stderr, SC_ACCT_OK);
			rc_log(LOG_NOTICE, "accounting OK, type %s, username %s, service %s%s%s",
				   type, username, service,(fproto)?"/":"", (fproto)?fproto:"");
		}
		else
		{
			fprintf(stderr, SC_ACCT_FAILED, result);
			rc_log(LOG_NOTICE, "accounting FAILED, type %s, username %s, service %s%s%s",
				   type, username, service,(fproto)?"/":"", (fproto)?fproto:"");
		}
		rc_avpair_free(send);
	}

	exit (result);
}
